@@ -5,7 +5,6 @@ from typing import Any, Optional, Union
 
 import blessed
 import rich
-from blessed import Terminal
 from rich.console import Console
 from rich.layout import Layout
 from rich.panel import Panel
@@ -18,37 +17,42 @@ from .filter_layout import FilterLayout
 from .help_layout import HelpLayout, HelpState, random_error_quote
 from .overview_layout import OverviewLayout, OverviewState, PreviewState
 from .stack_layout import StackFrame, StackLayout
+from .terminal import Terminal
 from .utils import is_selectable
 
-version = "1.4.4"
+version = "1.4.5"
 
-# TODO custom terminal class
+# TODO support ctrl-a + (whatever emacs keybinding to go to end of line)
+#  https://www.gnu.org/software/bash/manual/html_node/Commands-For-Moving.html
+# TODO add () to text for builtin-methods
+# TODO add a "searching..." title to search filter
+# TODO move stack/filter into the explorer object
+# TODO move help into the overview layout
 # TODO truncate public/private -> pub priv -> just public/private
-# TODO +-_= to change the size of the explorer window
-# TODO backspace to close filter/stack
+# TODO truncate explorer subtitle as well
 # TODO auto return on q or r
-# TODO builtin stack explorer? from objexplore import stackexplore
+# TODO builtin frame/stack explorer? from objexplore import stackexplore
 # TODO filter color the filters the same as the explorer
+# TODO Fix window when searching, then G, window does not move all the way to the bottom
 
 console = Console()
 
 
-class Explorer:
-    """ Explorer class used to interactively explore Python Objects """
+class ObjExploreApp:
+    """ Main Application class """
 
     def __init__(self, obj: Any, name_of_obj: str):
         cached_obj = CachedObject(obj, attr_name=name_of_obj)
         # Figure out all the attributes of the current obj's attributes
         cached_obj.cache()
 
-        # self.head_obj = cached_obj
         self.cached_obj: CachedObject = cached_obj
-        self.term = Terminal()
         self.stack = StackLayout(head_obj=self.cached_obj, visible=False)
-        self.help_layout = HelpLayout(version, visible=False, ratio=3)
-        self.explorer_layout = ExplorerLayout(cached_obj=cached_obj)
-        self.overview_layout = OverviewLayout(ratio=3)
-        self.filter_layout = FilterLayout(visible=False)
+        self.help_layout = HelpLayout(version, visible=False)
+        self.overview_layout = OverviewLayout()
+        self.term = Terminal(stack=self.stack)
+        self.explorer_layout = ExplorerLayout(cached_obj=cached_obj, term=self.term)
+        self.filter_layout = FilterLayout(term=self.term, visible=False)
 
         self.stack.append(
             StackFrame(
@@ -61,8 +65,8 @@ class Explorer:
         # Run self.draw() whenever the win change signal is caught
         try:
             signal.signal(signal.SIGWINCH, self.draw)
+        # Windows does not have SIGWINCH signal
         except AttributeError:
-            # OS does not have SIGWINCH signal
             pass
 
     def explore(self) -> Optional[Any]:
@@ -109,7 +113,7 @@ class Explorer:
                 self.filter_layout.backspace(self.cached_obj, self.explorer_layout)
             elif key.code == self.term.KEY_ESCAPE:
                 self.filter_layout.cancel_search(self.cached_obj)
-            elif key == "\n":
+            elif key.code == self.term.KEY_ENTER:
                 self.filter_layout.end_search(self.cached_obj)
             elif key.code == self.term.KEY_LEFT:
                 self.filter_layout.cursor_left()
@@ -130,7 +134,7 @@ class Explorer:
 
         if self.help_layout.visible:
             # Close help page
-            if key == "?" or key.code == self.term.KEY_ESCAPE:
+            if key == "?" or key.code in (self.term.KEY_ESCAPE, self.term.KEY_BACKSPACE):
                 self.help_layout.visible = False
                 return
 
@@ -160,11 +164,11 @@ class Explorer:
             else:
                 return
 
-        if self.help_layout.visible is False and key == "?":
+        if key == "?" and self.help_layout.visible is False:
             self.help_layout.visible = True
             return
 
-        # Views ###############################################################
+        # Stack ###############################################################
 
         if key == "o":
             if self.stack.visible:
@@ -175,9 +179,46 @@ class Explorer:
             else:
                 self.stack.set_visible()
 
-        elif key.code == self.term.KEY_ESCAPE and self.stack.visible:
+        elif (
+            key.code in (self.term.KEY_BACKSPACE, self.term.KEY_ESCAPE)
+            and self.stack.visible
+        ):
             self.stack.visible = False
 
+        elif (key == " " or key.code == self.term.KEY_ENTER) and self.stack.visible:
+            # If you are choosing the same frame as the current obj, then don't do anything
+            if self.stack[self.stack.index].cached_obj == self.cached_obj:
+                return
+            new_cached_obj = self.stack.select()
+            # TODO abstract the following
+            if not is_selectable(new_cached_obj.obj):
+                return
+            self.explorer_layout = ExplorerLayout(cached_obj=new_cached_obj, term=self.term)
+            self.cached_obj = new_cached_obj
+            self.cached_obj.cache()
+            self.filter_layout.cancel_search(self.cached_obj)
+            self.stack.append(
+                StackFrame(
+                    cached_obj=self.cached_obj,
+                    explorer_layout=self.explorer_layout,
+                    overview_layout=self.overview_layout,
+                )
+            )
+
+        elif (key == "j" or key.code == self.term.KEY_DOWN) and self.stack.visible:
+            self.stack.move_down(self.term.explorer_panel_height)
+
+        elif (key == "k" or key.code == self.term.KEY_UP) and self.stack.visible:
+            self.stack.move_up()
+
+        elif key == "g" and self.stack.visible:
+            self.stack.move_top()
+
+        elif key == "G" and self.stack.visible:
+            self.stack.move_bottom()
+
+        # Filter ##############################################################
+
         elif key == "n":
             if self.filter_layout.visible:
                 self.filter_layout.visible = False
@@ -187,12 +228,99 @@ class Explorer:
             else:
                 self.filter_layout.visible = True
 
-        elif key.code == self.term.KEY_ESCAPE and self.filter_layout.visible:
+        elif key == "/" and not self.filter_layout.receiving_input:
+            self.stack.visible = False
+            self.filter_layout.receiving_input = True
+            self.filter_layout.visible = True
+
+        elif (
+            key == " " or key.code == self.term.KEY_ENTER
+        ) and self.filter_layout.visible:
+            self.filter_layout.toggle(cached_obj=self.cached_obj)
+
+        elif (
+            key.code in (self.term.KEY_ESCAPE, self.term.KEY_BACKSPACE)
+        ) and self.filter_layout.visible:
             self.filter_layout.visible = False
 
+        elif (
+            key == "j" or key.code == self.term.KEY_DOWN
+        ) and self.filter_layout.visible:
+            self.filter_layout.move_down()
+
+        elif (
+            key == "k" or key.code == self.term.KEY_CODE
+        ) and self.filter_layout.visible:
+            self.filter_layout.move_up()
+
+        elif key == "g" and self.filter_layout.visible:
+            self.filter_layout.move_top()
+
+        elif key == "G" and self.filter_layout.visible:
+            self.filter_layout.move_bottom()
+
         elif key == "c":
             self.filter_layout.clear_filters(self.cached_obj)
 
+        # Explorer ############################################################
+
+        elif key == "k" or key.code == self.term.KEY_UP:
+            self.explorer_layout.move_up()
+
+        elif key == "j" or key.code == self.term.KEY_DOWN:
+            self.explorer_layout.move_down(
+                self.term.explorer_panel_height, self.cached_obj
+            )
+
+        elif key in ("l", " ") or key.code in (
+            self.term.KEY_ENTER,
+            self.term.KEY_RIGHT,
+            self.term.KEY,
+        ):
+            new_cached_obj = self.explorer_layout.selected_object
+            # TODO abstract the following
+            if not is_selectable(new_cached_obj.obj):
+                return
+
+            self.explorer_layout = ExplorerLayout(cached_obj=new_cached_obj, term=self.term)
+            self.cached_obj = new_cached_obj
+            self.cached_obj.cache()
+            self.filter_layout.cancel_search(self.cached_obj)
+            self.stack.append(
+                StackFrame(
+                    cached_obj=self.cached_obj,
+                    explorer_layout=self.explorer_layout,
+                    overview_layout=self.overview_layout,
+                )
+            )
+
+        elif key == "+":
+            self.term.increase_explorer_size()
+
+        elif key in ("-", "_"):
+            self.term.decrease_explorer_size()
+
+        elif key == "=":
+            self.term.explorer_size_extra = 0
+
+        # Go back to parent
+        elif (key == "h" or key.code == self.term.KEY_LEFT) and self.stack.stack:
+            self.stack.pop()
+            self.cached_obj = self.stack[-1].cached_obj
+            self.filter_layout.clear_filters(self.cached_obj)
+            self.explorer_layout = self.stack[-1].explorer_layout
+            self.overview_layout = self.stack[-1].overview_layout
+
+        elif key == "g":
+            self.explorer_layout.move_top()
+
+        elif key == "G":
+            self.explorer_layout.move_bottom(
+                self.term.explorer_panel_height, self.cached_obj
+            )
+
+        # Overview ############################################################
+
         # Switch between public and private attributes
         elif key in ("[", "]"):
             if self.explorer_layout.state == ExplorerState.public:
@@ -248,7 +376,7 @@ class Explorer:
             pydoc.pager(str_out)
 
         elif key == "H":
-            help(self.explorer_layout.selected_object)
+            help(self.explorer_layout.selected_object.obj)
 
         elif key == "i":
             with console.capture() as capture:
@@ -268,80 +396,12 @@ class Explorer:
             str_out = capture.get()
             pydoc.pager(str_out)
 
-        # Navigation ##########################################################
-
-        if key == "k" or key.code == self.term.KEY_UP:
-            if self.stack.visible:
-                self.stack.move_up()
-            elif self.filter_layout.visible:
-                self.filter_layout.move_up()
-            else:
-                self.explorer_layout.move_up()
-
-        elif key == "j" or key.code == self.term.KEY_DOWN:
-            if self.stack.visible:
-                self.stack.move_down(self.panel_height)
-            elif self.filter_layout.visible:
-                self.filter_layout.move_down()
-            else:
-                self.explorer_layout.move_down(self.panel_height, self.cached_obj)
-
-        elif key in ("\n", " ") and self.filter_layout.visible:
-            self.filter_layout.toggle(cached_obj=self.cached_obj)
-
-        elif key in ("\n", "l", " ") or key.code == self.term.KEY_RIGHT:
-
-            if self.stack.visible:
-                # If you are choosing the same frame as the current obj, then don't do anything
-                if self.stack[self.stack.index].cached_obj == self.cached_obj:
-                    return
-                new_cached_obj = self.stack.select()
-
-            elif self.filter_layout.visible:
-                return
-
-            else:
-                new_cached_obj = self.explorer_layout.selected_object
-                if not is_selectable(new_cached_obj.obj):
-                    return
-
-            self.explorer_layout = ExplorerLayout(cached_obj=new_cached_obj)
-            self.cached_obj = new_cached_obj
-            self.cached_obj.cache()
-            self.filter_layout.cancel_search(self.cached_obj)
-            self.stack.append(
-                StackFrame(
-                    cached_obj=self.cached_obj,
-                    explorer_layout=self.explorer_layout,
-                    overview_layout=self.overview_layout,
-                )
-            )
-
-        # go up
-        elif (key == "h" or key.code == self.term.KEY_LEFT) and self.stack:
-            self.stack.pop()
-            self.cached_obj = self.stack[-1].cached_obj
-            self.filter_layout.clear_filters(self.cached_obj)
-            self.explorer_layout = self.stack[-1].explorer_layout
-            self.overview_layout = self.stack[-1].overview_layout
-
-        elif key == "g":
-            self.explorer_layout.move_top()
-
-        elif key == "G":
-            self.explorer_layout.move_bottom(self.panel_height, self.cached_obj)
-
         # Other ################################################################
 
         # Return selected object
         elif key == "r":
             return self.explorer_layout.selected_object.obj
 
-        elif key == "/" and not self.filter_layout.receiving_input:
-            self.stack.visible = False
-            self.filter_layout.receiving_input = True
-            self.filter_layout.visible = True
-
     def get_explorer_layout(self) -> Layout:
         if self.stack.visible:
             layout = Layout()
@@ -398,7 +458,7 @@ class Explorer:
             subtitle=(
                 "[red][u]q[/u]:quit[/red] "
                 f"[cyan][u]?[/u]:{'exit ' if self.help_layout.visible else ''}help[/] "
-                "[white][dim][u]o[/u]:stack [u]n[/u]:filter [u]/[/u]:search [u]r[/u]:return"
+                "[bright_blue][u]o[/u]:stack [/bright_blue][bright_magenta][u]n[/u]:filter [/bright_magenta][aquamarine1][u]/[/u]:search [/aquamarine1][u]r[/u]:return"
             ),
             subtitle_align="left",
             height=self.term.height - 1,
@@ -406,26 +466,21 @@ class Explorer:
         )
         rich.print(object_explorer, end="")
 
-    @property
-    def panel_height(self) -> int:
-        # TODO think about putting this somewhere else?
-        # TODO maybe a custom terminal object
-        if self.stack.visible:
-            return (self.term.height - 10) // 2
-        else:
-            return self.term.height - 6
-
 
 def explore(obj: Any) -> Any:
     """ Run the explorer on the given object """
+    # Get the name of the variable sent to this function
+    # If someone calls this function like:
+    # >>> df = pandas.DataFrame()
+    # >>> explore(df)
+    # Then we want to extract `name` == 'df'
+    frame = inspect.currentframe()
+    name = frame.f_back.f_code.co_names[1]  # type: ignore
+    app = ObjExploreApp(obj, name_of_obj=name)
     try:
-        # Get the name of the variable sent to this function
-        # TODO and document
-        frame = inspect.currentframe()
-        name = frame.f_back.f_code.co_names[1]  # type: ignore
-        e = Explorer(obj, name_of_obj=name)
-        return e.explore()
+        return app.explore()
     except Exception as err:
+        print(app.term.move_down(app.term.height))
         console.print_exception(show_locals=True)
         print()
         rich.print(f"[red]{random_error_quote()}")
